File:    PRNSTR.ZIP
Date:    10/10/92
Author:  Patrick Reilly, 70274,161
Version: 1.01

	This file contains a stream class for BC++ that supports printers. It is
uploaded by the author and released to the public domain. Just for those who
are curious: the printer stream class is called 'prnstream' instead of
'pstream' because the latter is used in Borland's application framework
Turbo Vision.

CHANGES:
	This version (1.01) contains a few modifications (and one bug fix) from
the previous version.
	Bug fix. There was a bug in the first version where the allocated buffer
for the stream was not being deallocated. This was fixed by changing the
setbuf() calls to setb() (the original intention). Using setb() forces
streambuf to set the allocation flag (alloc_) to true so that ~streambuf()
deallocates the buffer. Thanks to Bill Oliver for pointing this out.
	Modification. The original version of PRNSTR.ZIP was ambiguous about
sending STATUS versus INIT in the constructors to check whether a printer
port is valid (again, thanks to Bill Oliver for pointing this out). This was
resolved in the following manner: ALL constructors use the biosprint() call
to send the STATUS check to the printer. The reasoning is that you may not
want the side-effects of using an INIT call - ie resetting printer settings,
ejecting the page, etc. As the test program shows, if you want the printer
to be initialized (whether after construction or at any time), use the
init() method.
	Modification. The printer status flags are now in the header file
(pstream.h) for easy access. They're use is demonstrated in the test program.
	Docs. The documentation has improved. First, there is now this read me
file <g>. Second, the header file PSTREAM.H now has a LOT more comments in
it, so you can get a better picture of how prnstream works. In particular,
this makes it easier to modify/derive from prnstream. An example:
	You may want to change prnstream to allow setting BOLD type. You MIGHT
consider a global function:
		prnstream& bold( prnstream& ps )
			{
			ps << "\x1B(s3B";
			return ps;
			}
		prnstream& normal( prnstream& ps )
			{
			ps << "\x1B(s0B";
			return ps;
			}
		...
		prnstream str(0);
		...
		str << bold << "This is Bold! " << normal << "This isn't!" << endl;

The problem with this scheme is that it ONLY works for HPPCL printers - ie
it WONT set an epson printer to bold/normal. A better solution would be
something like:
		class Printer : public prnstream
			{
			public:
				// ... include the ctors

				virtual void setBold() = 0;
				virtual void setNormal() = 0;
				virtual void setItalics() = 0;
				//... any others
			};

		class HPPrinter : public Printer
			{
			public:
				// ... include the ctors

				virtual void setBold()
					{
					*this << "\x1B(s3B";
					}
				virtual void setNormal()
					{
					*this << "\x1B(s0b0S";
					}
				virtual void setItalics()
					{
					*this << "\x1B(s1S";
					}
			};

		Printer& bold( Printer& prn )
			{
			prn.setBold();
			}
		Printer& normal( Printer& prn )
			{
			prn.setNormal();
			}
		Printer& italics( Printer& prn )
			{
			prn.setItalics();
			}

NOW you can just use the class for a particular printer like:
	Printer *pprn;
	if( ... )
		pprn = new HPPrinter(0);
	else
		pprn = new EpsonPrinter(0);
	...
	*pprn << bold << "This is BOLD," << italics << "This is Bold Italics".
	...
	delete pprn;

and (as long as each class' setBold() etc method is correct) you can change
font styles in a clean object-oriented manner.

	Notes: the class prnbuf (used internally by prnstream) uses the RTL
function biosprint() to write to the printer. There are 2 reasons for this:
1) DOS uses a quasi-'cooked' mode by default for the stdprn handle, since its
handled as just another file handle (ie ^Z (end-of-file) is ignored) which
can seriously mess up binary data (ie graphics) sent to the printer. You can
use the IOCTL functions to set stdprn to 'raw' mode, but this requires a bit
of knowledge on your part. Using biosprint(), the printer is ALWAYS in 'raw'
mode - prnbuf() itself does any 'cooking' required. This is easily changed
by the inherited method prnstreambase::setBinary(int). 2) When an error
occurs using stdprn, the infamous "Abort, Retry, ..." message is popped up
into the middle of your application (in TV, the status line pops up the
message). This is inconvenient at best, and makes the stream behave in a
manner different from other streams. By using BIOS, prnbuf() checks for
errors itself, and if any occur the ios state flags are changed to reflect
this. This makes prnstream behave like standard stream classes.
	Pat Reilly
	70274,161
